// Copyright HTC Corporation. All Rights Reserved.

#include "ISettingsModule.h"
#include "ISettingsSection.h"
#include "WaveOpenXRSettings.h"

#define LOCTEXT_NAMESPACE "WaveOpenXRSettings"

class FWaveOpenXRSettingsModule : public IModuleInterface
{
	virtual void StartupModule() override
	{
		RegisterSettings();
	}

	virtual void ShutdownModule() override
	{
		if (UObjectInitialized())
		{
			UnregisterSettings();
		}
	}

private:
	// Callback for when the settings were saved.
	bool HandleSettingsSaved()
	{
		UWaveOpenXRSettings* Settings = GetMutableDefault<UWaveOpenXRSettings>();
		bool isValidate = false;
		// Do validate here
		isValidate = true;

		if (isValidate)
			Settings->SaveConfig();

		return true;
	}

	void RegisterSettings()
	{
		if (ISettingsModule* SettingsModule = FModuleManager::GetModulePtr<ISettingsModule>("Settings"))
		{
			// Register the settings
			ISettingsSectionPtr SettingsSection = SettingsModule->RegisterSettings("Project", "Plugins", "WaveOpenXR",
				LOCTEXT("RuntimeGeneralSettingsName", "VIVE Wave OpenXR - Android"),
				LOCTEXT("RuntimeGeneralSettingsDescription", "Configurations for VIVE Wave OpenXR Extensions."),
				GetMutableDefault<UWaveOpenXRSettings>()
			);

			// Register the save handler to your settings, you might want to use it to
			// validate those or just act to settings changes.
			if (SettingsSection.IsValid())
			{
				SettingsSection->OnModified().BindRaw(this, &FWaveOpenXRSettingsModule::HandleSettingsSaved);
			}
		}
	}

	void UnregisterSettings()
	{
		// Ensure to unregister all of your registered settings here, hot-reload would
		// otherwise yield unexpected results.

		if (ISettingsModule* SettingsModule = FModuleManager::GetModulePtr<ISettingsModule>("Settings"))
		{
			SettingsModule->UnregisterSettings("Project", "Plugins", "WaveOpenXR");
		}
	}
};

IMPLEMENT_MODULE(FWaveOpenXRSettingsModule, WaveOpenXREditor);

#undef LOCTEXT_NAMESPACE
