// Copyright HTC Corporation. All Rights Reserved.

#include "WaveOpenXRHMD.h"

DEFINE_LOG_CATEGORY(LogWaveOpenXRHMD);

bool FWaveOpenXRHMD::GetCustomLoader(PFN_xrGetInstanceProcAddr* OutGetProcAddr)
{
	return false;
}

bool FWaveOpenXRHMD::IsStandaloneStereoOnlyDevice()
{
#if PLATFORM_ANDROID
	return true;
#else
	return false;
#endif
}

bool FWaveOpenXRHMD::GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR GetRequiredExtensions"));
	return true;
}

bool FWaveOpenXRHMD::GetInteractionProfile(XrInstance InInstance, FString& OutKeyPrefix, XrPath& OutPath, bool& OutHasHaptics)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR GetInteractionProfile"));
	return true;
}

bool FWaveOpenXRHMD::GetSpectatorScreenController(FHeadMountedDisplayBase* InHMDBase, TUniquePtr<FDefaultSpectatorScreenController>& OutSpectatorScreenController)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR GetSpectatorScreenController"));
#if PLATFORM_ANDROID
	OutSpectatorScreenController = nullptr;
	return true;
#else
	OutSpectatorScreenController = MakeUnique<FDefaultSpectatorScreenController>(InHMDBase);
	return false;
#endif
}

void FWaveOpenXRHMD::AddActions(XrInstance Instance, TFunction<XrAction(XrActionType InActionType, const FName& InName, const TArray<XrPath>& InSubactionPaths)> AddAction)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR AddActions"));
	return;
}

void FWaveOpenXRHMD::OnEvent(XrSession InSession, const XrEventDataBaseHeader* InHeader)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnEvent"));
	return;
}

const void* FWaveOpenXRHMD::OnCreateInstance(class IOpenXRHMDModule* InModule, const void* InNext)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnCreateInstance"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnGetSystem(XrInstance InInstance, const void* InNext)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnGetSystem"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnCreateSession(XrInstance InInstance, XrSystemId InSystem, const void* InNext)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnCreateSession"));
#if PLATFORM_ANDROID
	if (GRHISupportsRHIThread && GIsThreadedRendering && GUseRHIThread_InternalUseOnly)
	{
		SetRHIThreadEnabled(false, false);
	}
#endif
	return InNext;
}

const void* FWaveOpenXRHMD::OnBeginSession(XrSession InSession, const void* InNext)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnBeginSession"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnBeginFrame(XrSession InSession, XrTime DisplayTime, const void* InNext)
{
	//UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnBeginFrame"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnBeginProjectionView(XrSession InSession, int32 InLayerIndex, int32 InViewIndex, const void* InNext)
{
	//UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnBeginProjectionView"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnBeginDepthInfo(XrSession InSession, int32 InLayerIndex, int32 InViewIndex, const void* InNext)
{
	UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnBeginDepthInfo"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnEndProjectionLayer(XrSession InSession, int32 InLayerIndex, const void* InNext, XrCompositionLayerFlags& OutFlags)
{
	//UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnEndProjectionLayer"));
	OutFlags |= XR_COMPOSITION_LAYER_CORRECT_CHROMATIC_ABERRATION_BIT;
	return InNext;
}

const void* FWaveOpenXRHMD::OnEndFrame(XrSession InSession, XrTime DisplayTime, const TArray<XrSwapchainSubImage> InColorImages, const TArray<XrSwapchainSubImage> InDepthImages, const void* InNext)
{
	//UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnEndFrame"));
	return InNext;
}

const void* FWaveOpenXRHMD::OnSyncActions(XrSession InSession, const void* InNext)
{
	//UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR OnSyncActions"));
	return InNext;
}

void FWaveOpenXRHMD::PostSyncActions(XrSession InSession)
{
	//UE_LOG(LogWaveOpenXRHMD, Log, TEXT("Wave OpenXR PostSyncActions"));
	return;
}

IMPLEMENT_MODULE(FWaveOpenXRHMD, WaveOpenXRHMD)
