// Copyright HTC Corporation. All Rights Reserved.

using System;
using System.IO;
using EpicGames.Core; //Log

namespace UnrealBuildTool.Rules
{
	public class WaveOpenXRHMD : ModuleRules
	{
		public WaveOpenXRHMD(ReadOnlyTargetRules Target) : base(Target)
		{
			Log.TraceInformation("Execute WaveOpenXRHMD.Build.cs");
			PCHUsage = ModuleRules.PCHUsageMode.UseExplicitOrSharedPCHs;

			var EngineDir = Path.GetFullPath(Target.RelativeEnginePath);

			PrivateIncludePaths.AddRange(
				new string[] {
					//OpenXR/Source/OpenXRHMD/Public/OpenXRPlatformRHI.h need them.
					Path.Combine(EngineDir, @"Source/Runtime/OpenGLDrv/Private"), //OpenGLDrvPrivate.h
					Path.Combine(EngineDir, @"Source/Runtime/VulkanRHI/Private"), //VulkanRHIPrivate.h
				}
			);

			PrivateDependencyModuleNames.AddRange(
			new string[]
			{
				"Core",
				"CoreUObject",
				"Engine",
				"HeadMountedDisplay",
				"OpenGLDrv",
				"VulkanRHI",
				"OpenXRHMD",
				"WaveOpenXRLoader",
				"InputCore",
				"UMG",
			});

			PublicDependencyModuleNames.AddRange(
				new string[]
				{
					"EyeTracker",
				}
			);

			AddEngineThirdPartyPrivateStaticDependencies(Target, "Vulkan"); //VulkanRHI/Private/VulkanLoader.h need it.

			if (Target.bBuildEditor == true)
			{
				PrivateDependencyModuleNames.Add("UnrealEd");
			}

			if (Target.Platform == UnrealTargetPlatform.Win64)
			{
				PrivateIncludePaths.AddRange(
					new string[] {
						Path.Combine(EngineDir, @"Source/Runtime/Windows/D3D11RHI/Private"),
						Path.Combine(EngineDir, @"Source/Runtime/Windows/D3D11RHI/Private/Windows"),
						Path.Combine(EngineDir, @"Source/Runtime/D3D12RHI/Private"),
						Path.Combine(EngineDir, @"Source/Runtime/VulkanRHI/Private/Windows"),
					}
				);

				PrivateDependencyModuleNames.AddRange(
					new string[]
					{
						"D3D11RHI",
						"D3D12RHI",
					}
				);

				AddEngineThirdPartyPrivateStaticDependencies(Target, "NVAftermath");
				AddEngineThirdPartyPrivateStaticDependencies(Target, "IntelMetricsDiscovery");
				AddEngineThirdPartyPrivateStaticDependencies(Target, "IntelExtensionsFramework");
				AddEngineThirdPartyPrivateStaticDependencies(Target, "DX12");
				AddEngineThirdPartyPrivateStaticDependencies(Target, "OpenGL");
			}
			else if (Target.Platform == UnrealTargetPlatform.Android)
			{
				PrivateIncludePaths.Add(Path.Combine(EngineDir, @"Source/Runtime/VulkanRHI/Private/Android"));
				{
					string PluginPath = Utils.MakePathRelativeTo(ModuleDirectory, Target.RelativeEnginePath);
					AdditionalPropertiesForReceipt.Add("AndroidPlugin", Path.Combine(PluginPath, "WaveOpenXR_UPL.xml"));
				}
			}
		}
	}
}
