// Copyright HTC Corporation. All Rights Reserved.

#include "WaveOpenXRHandInteractionBP.h"
#include "WaveOpenXRHandInteraction.h"

TSharedPtr< class FWaveOpenXRHandInteraction, ESPMode::ThreadSafe > GetHandInteraction()
{
	TArray<FWaveOpenXRHandInteractionModule*> HandInteractions = IModularFeatures::Get().GetModularFeatureImplementations<FWaveOpenXRHandInteractionModule>(FWaveOpenXRHandInteractionModule::GetModularFeatureName());
	for (auto HandInteraction : HandInteractions)
	{
		return HandInteraction->CreateHandInteraction();
	}

	return nullptr;
}

float UWaveOpenXRHandInteractionBP::GetPinchStrength(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchStrength(isLeft);

	return 0;
}

FQuat UWaveOpenXRHandInteractionBP::GetPinchRotation(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchRotation(isLeft);

	return FQuat::Identity;
}

FVector UWaveOpenXRHandInteractionBP::GetPinchPosition(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchPosition(isLeft);

	return FVector::ZeroVector;
}

bool UWaveOpenXRHandInteractionBP::GetPinchActive(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchActive(isLeft);

	return false;
}

bool UWaveOpenXRHandInteractionBP::GetPinchValid(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchValid(isLeft);

	return false;
}

bool UWaveOpenXRHandInteractionBP::GetPinchTracked(bool isLeft)
{
	auto HandInteraction = GetHandInteraction();
	if (HandInteraction != nullptr)
		return HandInteraction->GetPinchTracked(isLeft);

	return false;
}
