// Copyright HTC Corporation. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "Modules/ModuleManager.h"
#include "Modules/ModuleInterface.h"
#include "Features/IModularFeatures.h"
#include "Features/IModularFeature.h"

/**
 * The public interface to this module.  In most cases, this interface is only public to sibling modules
 * within this plugin.
 */
class IWaveOpenXRHandInteractionModule : public IModuleInterface, public IModularFeature
{
public:
	/** Returns modular feature name for this module */
	static FName GetModularFeatureName()
	{
		static const FName HandInteractionModularFeatureName(TEXT("WaveHandInteraction"));
		return HandInteractionModularFeatureName;
	}

	/**
	* Singleton-like access to this module's interface.  This is just for convenience!
	* Beware of calling this during the shutdown phase, though. Your module might have been
	* unloaded already.
	*
	* @return Returns singleton instance, loading the module on demand if needed
	*/
	static inline IWaveOpenXRHandInteractionModule& Get()
	{
		return FModuleManager::LoadModuleChecked<IWaveOpenXRHandInteractionModule>("WaveOpenXRHandInteraction");
	}

	/**
	* Checks to see if this module is loaded and ready.  It is only valid to call Get() if
	* IsAvailable() returns true.
	*
	* @return True if the module is loaded and ready to use
	*/
	static inline bool IsAvailable()
	{
		return FModuleManager::Get().IsModuleLoaded("WaveOpenXRHandInteraction");
	}

	/**
	* Register module as an Hand Interaction on startup.
	*/
	virtual void StartupModule() override
	{
		IModularFeatures::Get().RegisterModularFeature(GetModularFeatureName(), this);
	}
};
