// Copyright HTC Corporation. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "Components/SceneComponent.h"

#include "Particles/ParticleSystemComponent.h"
#include "Particles/ParticleSystem.h"
#include "Math/Color.h"
#include "Engine/StaticMesh.h"
#include "Components/WidgetInteractionComponent.h"

#include "WaveOpenXRHandPointer.generated.h"

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class UWaveOpenXRHandPointer : public USceneComponent
{
	GENERATED_BODY()

public:	
	UWaveOpenXRHandPointer();

protected:
	virtual void BeginPlay() override;

public:	
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

private:
	uint32_t logFrame = 0;
	const uint32_t klogFrameCount = 300;
	bool printIntervalLog = false;
	void CheckLogInterval();

#pragma region
protected:
	bool m_IsLeft = false;
	/** Set up the tracking hand. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	bool IsLeft = false;

	/** Set up the available distance of hand LineTrace. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	float TraceDistance = 10000.f;

	float m_PinchOnThreshold = 0.7f;
	/** When the Pinch strength value is bigger than the Pinch On Threshold, the WaveOpenXRHandPointer will start sending events. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction", meta = (UIMin = "0.0", UIMax = "1.0"))
	float PinchOnThreshold = 0.7f;

	/** Set up the color of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	FLinearColor FadingParticleColor = FLinearColor::Green;
	/** Set up the EmitterIndex of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	int BeamEmitterIndex = 0;
	/** Set up the SourceIndex of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	int BeamSourceIndex = 0;
	/** Set up the TargetIndex of beam generated by Particle. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	int BeamTargetIndex = 0;
	/** Set up the vector parameter of the Particle's size. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	FVector FadingParticleSize = FVector(1, 1, 1);

	/** Set up the scale of the controller spot. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	FVector SpotScale = FVector(0.015f, 0.015f, 0.015f);
	/** Set up the rotation offset of the controller spot. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	FRotator SpotRotationOffset = FRotator(0, 0, 0);

	/** Set up the VirtualUserIndex of the controller widget. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	int32 WidgetVirtualUserIndex = 0;
	/** Set up the PointerIndex of the controller widget. Note: the Virtual User Index should be different in both hand. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Wave|OpenXR|HandInteraction")
	int32 WidgetPointerIndex = 3;
#pragma endregion Customized Variables

public:
	void SetHand(bool isLeft);
	void SetWidgetPointerIndex(int32 index);
	void SetPinchOnThreshold(float strength);

private:
	void UpdateCustomizedVariables();

// Resources
protected:
	UParticleSystemComponent* FadingParticle;
	UStaticMeshComponent* SpotPointer;
	UStaticMeshComponent* SpotPointerBlue;
	UWidgetInteractionComponent* WidgetInteraction;

private:
	void ConfigureResources();
	void ActivateBeamPointer();
	bool IsHandInteractable();

private:
	UParticleSystem* fadingParticleSystemTemplate;
	const int fadingParticleMaterialIndex = 0;
	UMaterialInterface* fadingParticleMaterial;
	const FName k_InitialColor = FName(TEXT("ParticleInitialColor"));
	const FName k_InitialSize = FName(TEXT("ParticleInitialSize"));

	UStaticMesh* spotPointerMesh;
	UMaterialInterface* spotPointerMaterial;
	UStaticMesh* spotPointerBlueMesh;
	UMaterialInterface* spotPointerBlueMaterial;

// Trace
private:
	FVector worldLocation;
	FRotator worldRotation;

	void SetIgnoreActors(FCollisionQueryParams& collisionParams);

	FVector targetPoint;

	bool tracedPhysics;
	//FHitResult hit;
	TArray<FHitResult> outHits;
	AActor* focusActor;
	const float k_DefaultPointerDistance = 100;
	float targetDistance;

	bool tracedGraphics;
	FVector tracedWidgetPosition;

// Pinch
private:
	bool IsClicked();
	bool pressState;
};
