// Copyright HTC Corporation. All Rights Reserved.

#include "WaveOpenXRWristTrackerBP.h"
#include "WaveOpenXRWristTracker.h"

FWaveOpenXRWristTracker* GetWristTracker()
{
	TArray<FWaveOpenXRWristTrackerModule*> InputDeviceModules = IModularFeatures::Get().GetModularFeatureImplementations<FWaveOpenXRWristTrackerModule>(IInputDeviceModule::GetModularFeatureName());
	for (auto InputDeviceModule : InputDeviceModules)
	{
		if (InputDeviceModule->GetModuleKeyName().IsEqual(IWaveOpenXRWristTrackerModule::WaveOpenXRWristTrackerModularKeyName))
			return InputDeviceModule->GetWristTracker();
	}

	return nullptr;
}

FQuat UWaveOpenXRWristTrackerBP::GetTrackerRotation(bool isLeft)
{
	auto WristTracker = GetWristTracker();
	if (WristTracker != nullptr)
		return WristTracker->GetTrackerRotation(isLeft);

	return FQuat::Identity;
}

FVector UWaveOpenXRWristTrackerBP::GetTrackerPosition(bool isLeft)
{
	auto WristTracker = GetWristTracker();
	if (WristTracker != nullptr)
		return WristTracker->GetTrackerPosition(isLeft);

	return FVector::ZeroVector;
}

bool UWaveOpenXRWristTrackerBP::GetTrackerActive(bool isLeft)
{
	auto WristTracker = GetWristTracker();
	if (WristTracker != nullptr)
		return WristTracker->GetTrackerActive(isLeft);

	return false;
}

bool UWaveOpenXRWristTrackerBP::GetTrackerValid(bool isLeft)
{
	auto WristTracker = GetWristTracker();
	if (WristTracker != nullptr)
		return WristTracker->GetTrackerValid(isLeft);

	return false;
}

bool UWaveOpenXRWristTrackerBP::GetTrackerTracked(bool isLeft)
{
	auto WristTracker = GetWristTracker();
	if (WristTracker != nullptr)
		return WristTracker->GetTrackerTracked(isLeft);

	return false;
}

bool UWaveOpenXRWristTrackerBP::GetTrackerKeyDown(bool isLeft, TrackerButton key)
{
	auto WristTracker = GetWristTracker();
	if (WristTracker != nullptr)
		return WristTracker->GetTrackerKeyDown(isLeft, key);

	return false;
}
